# ------------------------------------------------------------------------------
# Compute maximum flow

# Ralf Borndoerfer
# 10/2014

# Discrete Optimization in Transportation
# FU Berlin
# WS 2014

# http://www.zib.de/borndoerfer/Homepage/ws14-2.html

# ------------------------------------------------------------------------------
# input files:

# edges.dat
# Edges of the graph. Format is "1s 2s", where
# 1s = start city node
# 2s = end city node
# The edges are directed and the data contains forward and backward directions.

# cap.dat
# Edge capacities, format is "1s 2s 3n", where
# 1s = start city node
# 2s = end city node
# 3n = edge capacity

# ------------------------------------------------------------------------------
# optimal solution:

# objective value:                                26530

# y$Ut$Asd                                         2459   (obj:-1)
# y$Ut$Apd                                        15967   (obj:0)
# y$Ut$Bd                                         17351   (obj:0)
# y$Apd$Ut                                        15967   (obj:0)
# y$Asd$Ut                                        19810   (obj:1)
# y$Asd$Shl                                        9179   (obj:1)
# y$Lls$Zl                                        15712   (obj:0)
# y$Shl$Gvc                                       14611   (obj:0)
# y$Shl$Gv                                        13455   (obj:0)
# y$Zl$Lls                                        15712   (obj:0)
# y$Rtd$Bd                                         9179   (obj:0)
# y$Gv$Shl                                        13455   (obj:0)
# y$Gv$Rtd                                         9179   (obj:0)
# y$Gvc$Shl                                        5432   (obj:0)
# y$Gvc$Gv                                         9179   (obj:0)

# ------------------------------------------------------------------------------
# sets:

# edges (directed!)
set E := { read "edges.dat" as "<1s,2s>" comment "#" };

# nodes
set N := proj(E,<1>) union proj(E,<2>);

# ------------------------------------------------------------------------------
# parameters:

# capacities
param c[E] := read "cap.dat" as "<1s,2s> 3n" comment "#";

# ------------------------------------------------------------------------------
# source and target:

param source := "Asd"; #"Gn";
param target := "Bd";  #"Mt";

# ------------------------------------------------------------------------------
# variables:

# y[u,v] travelers on edge <u,v>
var y[<u,v> in E] integer >= 0 <= c[u,v];


# ------------------------------------------------------------------------------
# objective: zero

maximize obj: sum <source,v> in E: y[source,v] - sum <u,source> in E: y[u,source];

# ------------------------------------------------------------------------------
# constraints

subto conservation:
	forall <v> in N without {source, target}:
	sum <u,v> in E: y[u,v] - sum <v,w> in E: y[v,w] == 0;
